"""
Created on 23.04.2018 14:00

Author: Kasper Emil Thorvaldsen

This script will create price data that can be further utilized in the optimization model.
The data is based on NordPool and Statnett data

Energy prices are from NordPool

Primary reserve capacity prices are from Statnett

Note that the probabilities used are random:
Original plan: X energy prices times Y reserve capacity prices equal X*Y possibilities
probability from p1[day1] -> p1[day2] is the highest.
All other are randomized.
This is done as making time series with existing data is a big task in itself.

Case the author (Kasper) is using: NO3 (Trondheim), 30.01.2017 to 26.02.2017
"""
"""
Modified by: Jon Hvideberg Holte
28.01.2020

The price data is stored in an Excel-file if not other is specified.
Note that Statnett has changed their data setup, the extraction has been changed slightly since Kasper's creation.
This is activated in this script by "new_format" in line 72.

"""

import numpy as np
import random
import time
import sys
import create_price_data_scripts

info = {}                   #Create dictionary to store information

#Store relevant information for computation
info["filename"] = "elspot-prices_2018_extract.csv" #File for energy price location
info["num_days"] = 7                                #Number of days we want to store sequentially
info["num_weeks"] = 15 #4                               #Number of weeks we want to store sequentially
info["num_hours"] = 24                              #Number of hours we want to store sequentially
info["start_day"] = "01.01.2018" #"08.10.2018"                    #Starting day. The day we start to store data.
info["col_extract"] = 15                            #[15 is for NO3] What column in the csv-file we store data [value-1 due to python having 0 as first element]
info["param"] = "Eprice"                            #What is the stored values parameter
info["new_format"] = False
  
random.seed(15041994)



"""
Note, that for storing data:
- Starting day is when we start storing data. With the example up above, the following is happening:
- We store data from 30.01, all 24 hours, 7 days per week, 4 weeks.
"""

"""Extracting the energy prices (from NordPool) (some parameters are defined above)"""
Eprice = create_price_data_scripts.Create_price_datafiles(info)      #We extract data from energy price file

info["Eprice"] = Eprice                                             #Store the data in the dictionary


"""Extracting the reserve capacity prices (from Statnett)"""
info["filename"] = "PrimaryReservesDay_2018_extract.csv"            #New data file we extract data from

info["col_extract"] = 4 #11    #The prices are located in the 4th column in the new format  #NO3 is located in the 11th column in old format

info["param"] = "Cprice"                                            #What is the stored values parameter

info["new_format"] = True                           #We need an special way to extract the data from Statnett. They have changed their format since Kasper made this code

Cprice = create_price_data_scripts.Create_price_datafiles(info)              #Run the script, extract the reserve capacity price file

info["Cprice"] = Cprice                                             #Store the price data


#creating probability
#Litt vanskelig å skjønne hvordan sansynligheten blir opprettet
prob_matrix = np.zeros((info["num_weeks"]*info["num_weeks"],info["num_weeks"]*info["num_weeks"],info["num_days"]))  #Create matrix

for pr1 in range(0,info["num_weeks"]*info["num_weeks"]):        #For each current scenario
    for pr2 in range(0,info["num_weeks"]*info["num_weeks"]):    #For each possible next scenario
        for day in range(0,info["num_days"]):                   #For each day
            if pr1 == pr2:                                      #If the same scenario would occur from day1 to day2
                prob_matrix[pr1,pr2,day] = 100                  #Give that transition a high probability
            else:
                prob_matrix[pr1,pr2,day] = random.randint(1,75) #Rest gets a random probability lower than for equal
                
for day in range(0,info["num_days"]):                           #For each day
    for pr1 in range(0,info["num_weeks"]*info["num_weeks"]):    #For each current scenario
        cummulative = 0                                         #Store a cummulative value

        for pr2 in range(0,info["num_weeks"]*info["num_weeks"]):#For each possible next scenario
            cummulative = cummulative + prob_matrix[pr1,pr2,day]#Sum up the cummulative value
        
        for pr2 in range(0,info["num_weeks"]*info["num_weeks"]):#For each possible next scenario
            prob_matrix[pr1,pr2,day] = round(prob_matrix[pr1,pr2,day]/cummulative,5)    #The probability is the weighted factor of the cummulative value

info["prob_matrix"] = prob_matrix               #Store the data in the dictionary

info["data_store"] = "pricemod"             #Specify name to store all data

store_in_excel = True                           #Specify if the script should store the data in an Excel file
info["store_in_excel"] = store_in_excel                  #If stored in an Excel file, the data is NOT stored in a .dat-file
create_price_data_scripts.store_price_data(info)         #Call upon script to store price data

"""
End of script
"""