"""
Created on 23.04.2018 14:20

Author: Kasper Emil Thorvaldsen

This file contains multiple functions that the file "create_price_data.py" uses.

The following functions are available:

-Create_price_datafiles:
Takes in a data folder and information about where to extract data.
Extracts wished data and stores it in a matrix.
Depending on amount of data the user wants to store.
Output value is the matrix.
This can be used to extract both Eprice and Cprice in the authors examples.

-store_price_data
Takes in a matrix with relevant information, and the matrices for Eprice, Cprice and Prob.
Stores the data in a way that it can be read by the authors main model.
Note, that the data is being stored such that each Eprice can experience each Cprice.
Data is being stored in a given data-file. (currently name: "pricemod.dat")

"""
"""
Modified by: Jon Hvideberg Holte
28.01.2020

The following functions are modified:

-Create_price_datafiles:
Since Statnett has changed their data setup, the extraction has been changed slightly since Kasper's creation.
The change is activated by the "key".

-store_price_data
Data is being stored in a given Excel-file instead of a .dat-file. (currently name: "pricemod.xlsx")
The user could chose to store the data in a .dat-file if wanted in "create_price_data.py".
"""


import numpy as np                                          #Import numpy for complex array handling
import random                                               #Import random for random elements
import time                                                 #Import to measure time usage
import sys                                                  #Used to allow exiting program (only used for testing)
from datetime import datetime                               #Used to find current time and date
import pandas as pd

def Create_price_datafiles(info):
     
    r_file      = open(info["filename"],"r")    #Open file containing the relevant price data
    lines       = r_file.readlines()            #Read lines from files and store them
    r_file.close()                              #Close file as data has been retreived
    
    extract_data = False                        #Boolean value used to determine type of action
    
    num_days = info["num_days"]                 #Local storage of number of days
    num_weeks = info["num_weeks"]               #Local storage of number of weeks
    num_hours = info["num_hours"]               #Local storage of number of hours

    price_list = np.zeros((num_days,num_weeks,num_hours))       #List containing stored data
    
    starting_day = info["start_day"]                            #Starting date
    extract_col = info["col_extract"]                           #Column we want to extract from
    
    curr_day = 0                #Local value used for iteration purposes
    curr_week = 0               #Local value used for iteration purposes
    curr_hours = 0              #Local value used for iteration purposes
    
    key = 0
    if info["new_format"] == True:  #We need an special way to extract the data from Statnett,
        key = 1                     #They have changed their format since Kasper made this code
    
    if key == 1:
        for line in lines:          #For each line in the file
            line_split = line.split(";")        #Split using semicolon -> the file is stored as a csv-file with the use of ;
            
            if extract_data == True and line_split[2][:3] == "NO3":                    #If extracting data
                element = line_split[extract_col-1]     #We get information from correct column
                new_element = ""                        #We create a new string
                for letter in element:                  #For each letter, we check if the data is stored using , instead of . and replace
                    if letter == ",":
                        new_element = new_element + "."
                    else:
                        new_element = new_element + letter
                
                price_list[curr_day,curr_week,curr_hours] = float(new_element)  #Store the data as a float
                
                curr_hours = curr_hours +1              #Increase hour done
                if curr_hours == num_hours:             #Check if the number of hours permitted has been reached
                    curr_day = curr_day+1  
                    curr_hours = 0
                if curr_day == num_days:                #Check if the number of days has been reached
                    curr_week = curr_week+1
                    curr_day = 0
                if curr_week == num_weeks:              #Check if the number of weeks has been reached
                    break
            
            elif line_split[0][:10] == starting_day and line_split[2][:3] == "NO3":    #We check if the line we currently are in is the start of our process
    
                extract_data = True                     #If so, the extract data boolean is true
                element = line_split[extract_col-1]     #And we extract data from this row
                new_element = ""
                for letter in element:
                    if letter == ",":
                        new_element = new_element + "."
                    else:
                        new_element = new_element + letter
                
                price_list[curr_day,curr_week,curr_hours] = float(new_element)
                
                curr_hours = curr_hours +1
      
    else:
        for line in lines:          #For each line in the file
            line_split = line.split(";")        #Split using semicolon -> the file is stored as a csv-file with the use of ;
            
            if extract_data == True:                    #If extracting data
                element = line_split[extract_col-1]     #We get information from correct column
                new_element = ""                        #We create a new string
                for letter in element:                  #For each letter, we check if the data is stored using , instead of . and replace
                    if letter == ",":
                        new_element = new_element + "."
                    else:
                        new_element = new_element + letter
                
                price_list[curr_day,curr_week,curr_hours] = float(new_element)  #Store the data as a float
                
                curr_hours = curr_hours +1              #Increase hour done
                if curr_hours == num_hours:             #Check if the number of hours permitted has been reached
                    curr_day = curr_day+1  
                    curr_hours = 0
                if curr_day == num_days:                #Check if the number of days has been reached
                    curr_week = curr_week+1
                    curr_day = 0
                if curr_week == num_weeks:              #Check if the number of weeks has been reached
                    break
            
            elif line_split[0][:10] == starting_day:    #We check if the line we currently are in is the start of our process
    
                extract_data = True                     #If so, the extract data boolean is true
                element = line_split[extract_col-1]     #And we extract data from this row
                new_element = ""
                for letter in element:
                    if letter == ",":
                        new_element = new_element + "."
                    else:
                        new_element = new_element + letter
                
                price_list[curr_day,curr_week,curr_hours] = float(new_element)
                
                curr_hours = curr_hours +1



    return(price_list)                              #Return price list

    
def store_price_data(info):
    """-----------------------------"""
    """STORE THE DATA IN A .dat FILE"""
    """-----------------------------"""
    if info["store_in_excel"] == False:
        
        num_days = info["num_days"]                     #Store day value locally for shorter names
        num_weeks = info["num_weeks"]                   #Same here
        num_hours = info["num_hours"]                   #And here
        
        created_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S') #Store current date
        
        w_file = open(info["data_store"]+".dat","w")                   #Open the file where the price data will be stored in
        w_file.write("#Created on "+ created_time+ "\n")        #Store the time the data is being stored
        
        w_file.write("Param "+"Eprice"+" := \n")                            #Initialize parameter Eprice
        w_file.write(str(num_days) + "," + str(num_weeks*num_weeks)+"\n")   #Store number of days and weeks^2
    
        for day in range(0,num_days):       #For each day
            for pr2 in range(0,num_weeks):                 #For probability from
                for pr in range(0,num_weeks):               #For each probability to
                    w_file.write("#[Day "+str(day+1) +","+ " week " + str(pr2+1)+"]\n") #Write explanation of current scenario for user to read
                    for n in range(0,num_hours):          #For each hour
                        w_file.write(str(info["Eprice"][day,pr2,n]).ljust(7))   #Write down the price value
                        w_file.write("        ")                                #have som spacing between values
                        
                    w_file.write("\n")  #New Line
            
        w_file.write(";\n")             #; -> Finished with parameter input
        
        w_file.write("Param "+"Cprice"+" := \n")        #Start with Cprice parameter
        w_file.write(str(num_days) + "," + str(num_weeks*num_weeks)+"\n")
    
        for day in range(0,num_days):           #For each day
            for pr2 in range(0,num_weeks):                 #For each probability from
                for pr in range(0,num_weeks):               #For each probability to
                    w_file.write("#[Day "+str(day+1) +","+ " week " + str(pr+1)+"]\n")  #Explanation for user to read
                    for n in range(0,num_hours):          #For each hour 
                        w_file.write(str(info["Cprice"][day,pr,n]).ljust(7))        #Write down the price value
                        w_file.write("        ")                                    #Have some spacing between values
                        
                    w_file.write("\n")      #New line
            
        w_file.write(";\n")                 #; -> finished with parameter input
    
        
        w_file.write("Param "+"Prob"+" := \n")          #Initialize parameter Prob
        w_file.write(str(num_days) + "," + str(num_weeks*num_weeks)+"\n")   #Data about number of scenarios
    
        for day in range(0,num_days):                   #For each day
            for pr2 in range(0,num_weeks*num_weeks):    #For each probability from
                w_file.write("#day " + str(day+1) + ", Prob " + str(pr2+1) + ", prob2 *\n")
                for pr in range(0,num_weeks*num_weeks): #For each probability to
                    w_file.write(str(info["prob_matrix"][pr2,pr,day]).ljust(7)) #Write down probability value
                    w_file.write("        ")                #Have some spacing
                        
                w_file.write("\n")  #New line
            
        w_file.write(";\n")         #; -> Finished with parameter
        w_file.close()              #Close file
        
        print("Prices and probabilities are now stored in the .dat-file: pricemod.dat")



    """---------------------------------"""
    """ STORE THE DATA IN A EXCEL FILE  """
    """---------------------------------"""
    if info["store_in_excel"] == True:
        
        #Decide what the name of the output file should be
        excel_file = info["data_store"]+".xlsx"
        
        # Create a Pandas Excel writer object using XlsxWriter as the engine. 
        writer = pd.ExcelWriter(excel_file, engine ='xlsxwriter')
    
        # Storing data about performance in separate woorksheets
        df_1 = pd.DataFrame(info["Eprice"].reshape(-1, info["num_hours"] , order='F'))
        df_2 = pd.DataFrame(info["Cprice"].reshape(-1, info["num_hours"] , order='F'))   #Store data for each parameter in a dataframe and write it to excel
        df_1.to_excel(writer, sheet_name = "Eprice")         
        df_2.to_excel(writer, sheet_name = "Cprice")         
            #Since the data in the dictonary is a 3d-array, it must be reshaped into the dataframe
            #It is reshaped into rows=days*week and columns=hours
        
            # Storing data about performance in separate woorksheets
        df = pd.DataFrame(info["prob_matrix"].reshape(-1, info["num_days"] , order='C'))   #Store data for each parameter in a dataframe and write it to excel
        df.to_excel(writer, sheet_name = "Probabilities")         
            #Since the data in the dictonary is a 3d-array, it must be reshaped into the dataframe
            #It is reshaped into rows=probability*probability and columns=days
        
        # Close the Pandas Excel writer object and output the Excel file. 
        writer.save() 
        
        print("Prices and probabilities in the Excel-file: " + excel_file)
        
    return
    
    
    return                      #Close script

    
    
